@@ -56,5 +56,5 @@ class Fuse(NumpyOp):
 
     def forward(self, data: List[np.ndarray], state: Dict[str, Any]) -> List[np.ndarray]:
         data = {key: elem for key, elem in zip(self.inputs, data)}
-        forward_numpyop(self.ops, data, state["mode"])
+        forward_numpyop(self.ops, data, state)
         return [data[key] for key in self.outputs]
