@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ==============================================================================
-from typing import Any, Dict, Iterable, List, TypeVar, Union
+from typing import Any, Dict, Iterable, List, Tuple, TypeVar, Union
 
 import tensorflow as tf
 import torch
@@ -38,9 +38,9 @@ class MeanSquaredError(LossOp):
         average_loss: Whether to average the element-wise loss after the Loss Op.
     """
     def __init__(self,
-                 inputs: Union[None, str, Iterable[str]] = None,
-                 outputs: Union[None, str, Iterable[str]] = None,
-                 mode: Union[None, str, Iterable[str]] = None,
+                 inputs: Union[Tuple[str, str], List[str]],
+                 outputs: str,
+                 mode: Union[None, str, Iterable[str]] = "!infer",
                  average_loss: bool = True):
         self.average_loss = average_loss
         super().__init__(inputs=inputs, outputs=outputs, mode=mode)
