@@ -32,7 +32,7 @@ from fastestimator.op.numpyop.meta.sometimes import Sometimes
 from fastestimator.op.numpyop.numpyop import NumpyOp, forward_numpyop
 from fastestimator.schedule.schedule import Scheduler, get_current_items
 from fastestimator.util.traceability_util import traceable
-from fastestimator.util.util import pad_batch, to_list, to_set
+from fastestimator.util.util import get_num_devices, pad_batch, to_list, to_set
 
 DataSource = TypeVar('DataSource', Dataset, DataLoader, tf.data.Dataset)
 
@@ -50,8 +50,8 @@ class Pipeline:
         ops: NumpyOps to be used for pre-processing. NOTE: This argument is only applicable when using a FastEstimator
             Dataset.
         num_process: Number of CPU threads to use for data pre-processing. NOTE: This argument is only applicable when
-            using a FastEstimator Dataset. None will default to the system CPU count. Multiprocessing can be disabled by
-            passing 0 here, which can be useful for debugging.
+            using a FastEstimator Dataset. None will default to min(n_cpus, max(32, 32*n_gpus)). Multiprocessing can be
+            disabled by passing 0 here, which can be useful for debugging.
         drop_last: Whether to drop the last batch if the last batch is incomplete.
         pad_value: The padding value if batch padding is needed. None indicates that no padding is needed. NOTE: This
             argument is only applicable when using a FastEstimator Dataset.
@@ -77,7 +77,7 @@ class Pipeline:
         if mp.get_start_method(allow_none=True) != 'fork':
             print("FastEstimator-Warn: Pipeline multiprocessing is disabled. OS must support the 'fork' start method.")
             num_process = 0
-        self.num_process = num_process if num_process is not None else os.cpu_count()
+        self.num_process = num_process if num_process is not None else min(os.cpu_count(), 32 * get_num_devices())
         self.drop_last = drop_last
         self.pad_value = pad_value
         self.collate_fn = collate_fn
