@@ -14,5 +14,5 @@
 # ==============================================================================
 from fastestimator.summary import logs
 from fastestimator.summary.history import connect, delete, HistoryRecorder, HistoryReader, update_settings
-from fastestimator.summary.summary import Summary
+from fastestimator.summary.summary import average_summaries, Summary, ValWithError
 from fastestimator.summary.system import System
