@@ -16,9 +16,11 @@ import time
 from collections import deque
 from typing import Iterable, List, Optional, Set, Union
 
+from natsort import humansorted
 import numpy as np
 
 from fastestimator.backend.get_lr import get_lr
+from fastestimator.summary.summary import ValWithError
 from fastestimator.summary.system import System
 from fastestimator.util.data import Data
 from fastestimator.util.traceability_util import traceable
@@ -210,6 +212,35 @@ class EvalEssential(Trace):
             data.write_with_log(key, np.mean(np.array(value_list), axis=0))
 
 
+@traceable()
+class TestEssential(Trace):
+    """A trace to collect important information during evaluation.
+
+    Please don't add this trace into an estimator manually. FastEstimator will add it automatically.
+
+    Args:
+        monitor_names: Any keys which should be collected over the course of an test epoch.
+    """
+    def __init__(self, monitor_names: Set[str]) -> None:
+        super().__init__(mode="test", inputs=monitor_names)
+        self.test_results = None
+
+    def on_epoch_begin(self, data: Data) -> None:
+        self.test_results = None
+
+    def on_batch_end(self, data: Data) -> None:
+        if self.test_results is None:
+            self.test_results = {key: [data[key]] for key in self.inputs if key in data}
+        else:
+            for key in self.inputs:
+                if key in data:
+                    self.test_results[key].append(data[key])
+
+    def on_epoch_end(self, data: Data) -> None:
+        for key, value_list in self.test_results.items():
+            data.write_with_log(key, np.mean(np.array(value_list), axis=0))
+
+
 @traceable()
 class Logger(Trace):
     """A Trace that prints log messages.
@@ -253,13 +284,20 @@ class Logger(Trace):
         if log_epoch:
             log_message += "epoch: {}; ".format(self.system.epoch_idx)
             self.system.write_summary('epoch', self.system.epoch_idx)
-        for key, val in data.read_logs().items():
-            val = to_number(val)
-            self.system.write_summary(key, val)
-            if val.size > 1:
-                log_message += "\n{}:\n{};".format(key, np.array2string(val, separator=','))
-            else:
+        deferred = []
+        for key, val in humansorted(data.read_logs().items(), key=lambda x: x[0]):
+            if isinstance(val, ValWithError):
                 log_message += "{}: {}; ".format(key, str(val))
+            else:
+                val = to_number(val)
+                if val.size > 1:
+                    deferred.append("\n{}:\n{};".format(key, np.array2string(val, separator=',')))
+                else:
+                    log_message += "{}: {}; ".format(key, str(val))
+            self.system.write_summary(key, val)
+        log_message = log_message.strip()
+        for elem in deferred:
+            log_message += elem
         print(log_message)
 
 
@@ -298,7 +336,7 @@ def sort_traces(traces: List[Trace], available_outputs: Optional[Set[str]] = Non
         trace = trace_deque.popleft()
         ins = set(trace.inputs)
         outs = set(trace.outputs)
-        if not ins or isinstance(trace, (TrainEssential, EvalEssential)):
+        if not ins or isinstance(trace, (TrainEssential, EvalEssential, TestEssential)):
             sorted_traces.append(trace)
             available_outputs |= outs
         elif "*" in ins:
