@@ -48,7 +48,15 @@ class SampleTensorOp(TensorOp):
         return data
 
 
+class PlusOneNumpyOp(NumpyOp):
+    def forward(self, data, state):
+        return data + 1
+
+
 def get_torch_lenet_model_weight(model):
+    if torch.cuda.device_count() > 1:
+        model = model.module
+
     weight = []
     weight.append(deepcopy(model.conv1.weight.data.numpy()))
     weight.append(deepcopy(model.conv2.weight.data.numpy()))
@@ -68,6 +76,9 @@ def get_tf_model_weight(model):
 
 
 def get_torch_one_layer_model_weight(model):
+    if torch.cuda.device_count() > 1:
+        model = model.module
+
     weight = []
     weight.append(deepcopy(model.fc1.weight.data.numpy()))
 
@@ -179,7 +190,7 @@ class TestNetworkBuildOptimizer(unittest.TestCase):
         str_list = ['adadelta', 'adagrad', 'adam', 'adamax', 'rmsprop', 'sgd']
         for opt_name in str_list:
             with self.subTest(optimizer_fn=opt_name):
-                optimizer = fe.network._build_optimizer(optimizer_fn=opt_name, model=self.tf_model, framework="tf")
+                optimizer = fe.network._build_optimizer(optimizer_fn=opt_name, model=self.tf_model, framework="tf", mixed_precision=False)
                 self.assertIsInstance(optimizer, tf.optimizers.Optimizer)
 
     def test_network_build_optimizer_torch_model_optimizer_str(self):
@@ -188,19 +199,26 @@ class TestNetworkBuildOptimizer(unittest.TestCase):
             with self.subTest(optimizer_fn=opt_name):
                 optimizer = fe.network._build_optimizer(optimizer_fn=opt_name,
                                                         model=self.torch_model,
-                                                        framework="torch")
+                                                        framework="torch",
+                                                        mixed_precision=False)
                 self.assertIsInstance(optimizer, torch.optim.Optimizer)
 
     def test_network_build_optimizer_tf_model_optimizer_fn(self):
         fn_list = [tf.optimizers.Adadelta, lambda: tf.optimizers.Adam(lr=0.001)]
         for opt_fn in fn_list:
             with self.subTest(optimizer_fn=opt_fn):
-                optimizer = fe.network._build_optimizer(optimizer_fn=opt_fn, model=self.tf_model, framework="tf")
+                optimizer = fe.network._build_optimizer(optimizer_fn=opt_fn,
+                                                        model=self.tf_model,
+                                                        framework="tf",
+                                                        mixed_precision=False)
                 self.assertIsInstance(optimizer, tf.optimizers.Optimizer)
 
     def test_network_build_optimizer_torch_model_optimizer_fn(self):
         opt_fn = lambda x: torch.optim.SGD(params=x, lr=0.01)
-        optimizer = fe.network._build_optimizer(optimizer_fn=opt_fn, model=self.torch_model, framework="torch")
+        optimizer = fe.network._build_optimizer(optimizer_fn=opt_fn,
+                                                model=self.torch_model,
+                                                framework="torch",
+                                                mixed_precision=False)
         self.assertIsInstance(optimizer, torch.optim.Optimizer)
 
 
@@ -222,7 +240,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                            optimizer_fn=optimizer,
                                            weight="example_path",
                                            name="test",
-                                           framework="tf")
+                                           framework="tf",
+                                           mixed_precision=False)
 
             _, weight = fake.call_args[0]
             self.assertEqual(weight, "example_path")
@@ -233,7 +252,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name="test",
-                                       framework="tf")
+                                       framework="tf",
+                                       mixed_precision=False)
 
         with self.subTest("check optimizer instantiation"):
             for optimizer in model.optimizer.get_all_values():
@@ -254,7 +274,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name=None,
-                                       framework="tf")
+                                       framework="tf",
+                                       mixed_precision=False)
 
         with self.subTest("check optimizer instantiation"):
             for optimizer in model.optimizer.get_all_values():
@@ -269,7 +290,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name=None,
-                                       framework="tf")
+                                       framework="tf",
+                                       mixed_precision=False)
         with self.subTest("check optimizer instantiation"):
             self.assertIsInstance(model.optimizer, tf.optimizers.Optimizer)
 
@@ -282,7 +304,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name=None,
-                                       framework="torch")
+                                       framework="torch",
+                                       mixed_precision=False)
 
         with self.subTest("check optimizer instantiation"):
             for optimizer in model.optimizer.get_all_values():
@@ -297,7 +320,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name=None,
-                                       framework="torch")
+                                       framework="torch",
+                                       mixed_precision=False)
 
         with self.subTest("check optimizer instantiation"):
             for optimizer in model.optimizer.get_all_values():
@@ -312,7 +336,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name=None,
-                                       framework="torch")
+                                       framework="torch",
+                                       mixed_precision=False)
         with self.subTest("check optimizer instantiation"):
             self.assertIsInstance(model.optimizer, torch.optim.Optimizer)
 
@@ -323,7 +348,8 @@ class TestNetworkFeCompile(unittest.TestCase):
                                        optimizer_fn=optimizer,
                                        weight=None,
                                        name=None,
-                                       framework="torch")
+                                       framework="torch",
+                                       mixed_precision=False)
 
 
 class TestNetworkBuild(unittest.TestCase):
@@ -393,11 +419,13 @@ class TestNetworkTransform(unittest.TestCase):
     def test_network_transform_one_layer_model_tf(self):
         model = fe.build(model_fn=one_layer_tf_model, optimizer_fn="adam")
         weight = get_tf_model_weight(model)
-        network = fe.Network(ops=[
-            ModelOp(model=model, inputs="x", outputs="y_pred"),
-            MeanSquaredError(inputs=("y_pred", "y"), outputs="ce"),
-            UpdateOp(model=model, loss_name="ce")
-        ])
+        network = fe.Network(
+            ops=[
+                ModelOp(model=model, inputs="x", outputs="y_pred"),
+                MeanSquaredError(inputs=("y_pred", "y"), outputs="ce"),
+                UpdateOp(model=model, loss_name="ce")
+            ],
+            pops=PlusOneNumpyOp(inputs="y_pred", outputs="y_pred_processed"))
         batch = {"x": np.array([[1, 1, 1]]), "y": np.array([[1]])}
         batch = network.transform(data=batch, mode="train")
 
@@ -405,6 +433,10 @@ class TestNetworkTransform(unittest.TestCase):
             ans = np.array([[6]], dtype=np.float32)  # 1*1 + 1*2 + 1*3
             self.assertTrue(np.array_equal(batch["y_pred"].numpy(), ans))
 
+        with self.subTest("postprocessing y_pred check"):
+            ans = np.array([[7]], dtype=np.float32)  # 1*1 + 1*2 + 1*3 + 1
+            self.assertTrue(np.array_equal(batch["y_pred_processed"], ans))
+
         with self.subTest("output ce check"):
             self.assertEqual(batch["ce"].numpy(), 25)  # (6-1)^2
 
@@ -415,11 +447,13 @@ class TestNetworkTransform(unittest.TestCase):
     def test_network_transform_one_layer_model_torch(self):
         model = fe.build(model_fn=OneLayerTorchModel, optimizer_fn="adam")
         weight = get_torch_one_layer_model_weight(model)
-        network = fe.Network(ops=[
-            ModelOp(model=model, inputs="x", outputs="y_pred"),
-            MeanSquaredError(inputs=("y_pred", "y"), outputs="ce"),
-            UpdateOp(model=model, loss_name="ce")
-        ])
+        network = fe.Network(
+            ops=[
+                ModelOp(model=model, inputs="x", outputs="y_pred"),
+                MeanSquaredError(inputs=("y_pred", "y"), outputs="ce"),
+                UpdateOp(model=model, loss_name="ce")
+            ],
+            pops=PlusOneNumpyOp(inputs="y_pred", outputs="y_pred_processed"))
         batch = {
             "x": np.array([[
                 1,
@@ -433,6 +467,10 @@ class TestNetworkTransform(unittest.TestCase):
             ans = np.array([[6]], dtype=np.float32)  # 1*1 + 1*2 + 1*3
             self.assertTrue(np.array_equal(batch["y_pred"].numpy(), ans))
 
+        with self.subTest("postprocessing y_pred check"):
+            ans = np.array([[7]], dtype=np.float32)  # 1*1 + 1*2 + 1*3 + 1
+            self.assertTrue(np.array_equal(batch["y_pred_processed"], ans))
+
         with self.subTest("output ce check"):
             self.assertEqual(batch["ce"].numpy(), 25)  # (6-1)^2
 
